-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
-- Script is the main part of the mission. You can't generally create a mission without some Lua
-- so get ready to some light to medium programming and add http://lua.org , Cortex Command Wiki
-- and forum search to the favourites in your browser.
--
-- This method\function is the place where the mission starts. Here you define and initialize variables, 
-- create scripted objects (brains, actors etc) and so on. If something goes wrong in this method, then
-- you kicked out to the campaign screen with error message in the console. Some times you need to restart
-- your mission twice, because CC will keep kicking you out complaining about already fixed errors.
-- Notice the name of the function, CampaignMission02 part is that Lua class name that you used in mission activities.ini
function CampaignMission02:StartActivity()
	-- Always print debugging information to the console but keep in mind that heavy prinintg
	-- causes heavy lag. Comment all the unnecessary prints when you release your mission to the public.
	-- If you notice some heavy unexpected lag, check the console. CC is probably flooding it with error messages.
	print("START! -- CampaignMission02:StartActivity()!");

	-- Array which stores flags for players that lost their brain.
	self.braindead = {};
	-- Array which stores brains assigned to players. It is used for quick enumeration of 
	-- playable brains to trigger something or show mission marks.
	self.AssignedBrains = {};
	-- This is used to limit the amount of ingame movable objects (MO's). Currently CC goes mad if this number exceeds
	-- 255. To avoid unexpected behaviour you MUST always take care of the number of the movable ID's. We'll talk more
	-- about movable objects and limits later when we'll discuss spawning. Further reading - http://www.datarealms.com/wiki/index.php/Scenes
	self.MOIDLimit = 200;
    
	-- Here we store the amount of gold that player will recieve at start. Note that we not actualyy give player
	-- real gold here
	self.PlayerGold = 0; -- Player starting credits
	
   	-- Team variables are used just for convenience
   	-- Team 2 is always CPU
   	self.CPUTeam = Activity.TEAM_2;
	self.PlayerTeam = Activity.TEAM_1;
	self.RogueTeam = -1;

	-- Setup mission objectives. Here we have a "phaseless" mission, so you can complete objectives in any order (well almost, except evacuate).
	-- I made four simple objectives here. Clear the area of enemies, Assassinate single target, Obtain special item and finally Evacuate. 
	-- Almost just like in Cortex Command Activity we'll use the same two common functions for objectives. That would be some simple "framework"
	-- for adding independent objectives with just a click of the mouse (just kidding, but certainly without much pain). Two common funcs for 
	-- an objective here are Init and Update. Some Objectives require Spawn function which respawns enemies, items etc. if necessary.
	self:ObjectiveSweepInit();
	self:ObjectiveAssassinateInit();
	self:ObjectiveGetChipInit();
	self:ObjectiveEvacuateInit();
	
   	-- Set up players
    -- Here we create brains for players. Although current CC don't have predefined activities with multiplayer missions
	-- keep in mind that this behaviour may change, so you should treat gameplay as it was a multiplayer game so you'll have
	-- to fix less when CC will support standard multiplayer activities for missions
	for player = 0, self.PlayerCount - 1 do
        -- Check if we already have a brain assigned
        if not self:GetPlayerBrain(player) then
			self.braindead[player] = false;
			-- Create brain here and alter all his neccessary properties
			local brain = CreateAHuman("Brain Robot" , "base.rte");
			-- Actor location on the scene. X=0 , Y=0 is the upper left corner of the map. Greater X is to the right.
			-- Greater Y is to the bottom.
			brain.Pos = Vector(150 + player * 30 , 550);
			-- Create weapon and give it to the actor.
			brain:AddInventoryItem(CreateHDFirearm("Assault Rifle" , "Coalition.rte"));
			brain:AddInventoryItem(CreateHDFirearm("Auto Shotgun" , "Coalition.rte"));
			-- Make brain just stay and do nothing. Further reading - http://www.datarealms.com/wiki/index.php/LuaDocs/Actor
			brain.AIMode = Actor.AIMODE_SENTRY;
			-- Add created brain to the array of assigned actors
			self.AssignedBrains[player] = brain;

            -- Let the spawn into the world, passing ownership
            MovableMan:AddActor(brain);
            -- Set the found brain to be the selected actor at start
            self:SetPlayerBrain(brain, player);
            SceneMan:SetScroll(self:GetPlayerBrain(player).Pos, player);
            -- Set the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
            self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player);
			-- Select brain as current actor
			self:SwitchToActor(brain, player, self.PlayerTeam);
        end
    end
	
	-- Here we spawn another friendly clone
	local clone = CreateAHuman("Soldier Light" , "Coalition.rte");
	clone.Pos = Vector(100 , 550);
	clone:AddInventoryItem(CreateHDFirearm("Assault Rifle" , "Coalition.rte"));
	clone:AddInventoryItem(CreateHDFirearm("Auto Shotgun" , "Coalition.rte"));
	clone.Team = self.PlayerTeam;
	clone.AIMode = Actor.AIMODE_SENTRY;
	MovableMan:AddActor(clone);
	
    -- Set the team's funds. Here we actually give a player some gold.
    self:SetTeamFunds(self.PlayerGold , self.PlayerTeam);
	
	self.GenericTimer = Timer();
	self.GenericTimer:Reset();	
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
-- This function is called when mission is paused
function CampaignMission02:PauseActivity(pause)
    print("PAUSE! -- CampaignMission02:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
-- This function is called after mission has ended
function CampaignMission02:EndActivity()
    print("END! -- CampaignMission02:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
-- This is the main loop of the mission. This function is called many times at a second and
-- processes all mission events. You should not write heavy code here, just call some specific functions
-- with actual mission code. Nobody can prevent you from writing all mission code here, but at some moment
-- you'll find that you missed some end somewhere, and you have to check 1000 linies of code to find out where,
-- because Lua gives you wring line numbers in error message
function CampaignMission02:UpdateActivity()
   	-- Clear all objective markers, they get re-added each frame
   	self:ClearObjectivePoints();
	
	-- Update all mission objectives. As all objectives are quite isolated all we need to add a new objective is to add
	-- one line at init, on line here and write these two functions.
	self:ObjectiveSweepUpdate();
	self:ObjectiveAssassinateUpdate();
	self:ObjectiveGetChipUpdate();
	self:ObjectiveEvacuateUpdate();
	
	-- Check for defeat conditions
	self:DoBrainSelection();
	
	if not self.GenericTimer:IsPastSimMS(500) then
		for player = 0, self.PlayerCount - 1 do
			FrameMan:SetScreenText("Mission 02", player, 100, 5000, false);
		end --for
	end
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
-- Here we check if we stil have a brain to cobtrol our troops. If not - game over
function CampaignMission02:DoBrainSelection()
	-- Check if mission is still running to prevent mission failure
	-- if you waste brain after victory or lose it's object after evacuation
	if not (self.ActivityState == Activity.OVER) then
		-- Iterate through all human players
		for player = 0, self.PlayerCount - 1 do
			-- The current player's team
			local team = self:GetTeamOfPlayer(player);
	
			-- Make sure the game is not already ending
			-- Check if any player's brain is dead
			local brain = self:GetPlayerBrain(player);
			-- Notice the MovableMan:IsActor(brain) function call. This is eesntial when you reference
			-- ingame objects especially actors from lua. It checks if onjects still exists. If you reference 
			-- properties of a non-existent actor (actor can die at any moment) the game will simply crash. 
			-- No fancy console errors at all, just a silent crash.
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				-- Reset player brain
				self:SetPlayerBrain(nil, player);
				-- Try to find a new unasigned brain this player can use instead, or if his old brain entered a craft
				local newBrain = MovableMan:GetUnassignedBrain(team);
				-- Found new brain actor, assign it and keep on truckin'
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
				else
					-- All brains of this team are dead. Show fail reason
					FrameMan:SetScreenText("Commander died!", player, 100, -1, false);
					-- Set the flag that this player is brainless
					self.braindead[player] = true;
					
					-- Now see if all brains of self player's team are dead, and if so, end the game
					if not MovableMan:GetClosestBrainActor(team) then
						-- Show Fail\Win banner to participating teams. 
						self.WinnerTeam = self:OtherTeam(team);
						-- Game over
						ActivityMan:EndActivity();
					end
				end
			else
				-- Update the observation target to the brain, so that if/when it dies, the view flies to it in 
				-- observation mode
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end --if
end
-----------------------------------------------------------------------------------------
-- Mission objective "Sweep" initialization
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveSweepInit()
	-- Try to provide some >limited< debugging output even in released missions. That will help you later.
	-- Don't forget that heavy prining means heavy laging.
	print ("Sweep Init");
	-- Typical init function resets winning condition variables, creates objects, resets timers etc.
	self.SweepZone = SceneMan.Scene:GetArea("SweepZone"); -- Zone to clear out of enemies
	self.SweepComplete = false;
	self.SweepSpawnTimer = Timer();
	
	for i = 0 , 3 do
		self:ObjectiveSweepSpawnEnemy();
	end -- for
end
-----------------------------------------------------------------------------------------
-- Mission objective "Sweep" update state
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveSweepUpdate()
	-- Only check winning conditions if this objective is not completed. Save as much CPU cycles as you can so you 
	-- can waste them somewhere else.
	if self.SweepComplete == false then
		-- In Sweep objective we need to count enemies in specific area. These areas are added in Area Editor.
		local liveenemies = 0;
		-- Why do we need two loops here you ask? That't quite tricky. The problem is that when you add actors in Init
		-- phase they are placed in MovableMan.AddedActors collection first and then after a short while they are 
		-- moved to MovableMan.Actors. If we enumerate only MovableMan.Actors we'll get winning condition triggered on
		-- start, because this collection will be empty. So we need to enumerate both of them. I can think of other 
		-- ways to avoid this. Check for winning conditions after a short while after start (250 ms is enough). Initialize
		-- objectives inside UpdateActivity function or maybe some others. But they are hacky enough to choose both 
		-- collection enumeration.
		for actor in MovableMan.AddedActors do
			-- Exclude doors. They are actors too.
			if actor.ClassName ~= "ADoor" and actor.Team == self.CPUTeam and self.SweepZone:IsInside(actor.Pos) then
				liveenemies = liveenemies + 1;
				self:AddObjectivePoint("Kill All", actor.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
			end
		end -- for
		
		for actor in MovableMan.Actors do
			if actor.ClassName ~= "ADoor" and actor.Team == self.CPUTeam and self.SweepZone:IsInside(actor.Pos) then
				liveenemies = liveenemies + 1;
				self:AddObjectivePoint("Kill All", actor.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
			end
		end
		
		-- Enemies counted, check completion conditions
		if liveenemies == 0 then
			print ("Sweep complete");
			self.SweepComplete = true;
		end -- if
		
		-- Spawn new enemies every 5 secs to make more fun
		if self.SweepSpawnTimer:IsPastSimMS(5000) then
			self:ObjectiveSweepSpawnEnemy()
			self.SweepSpawnTimer:Reset();
		end -- if
	end --if 
end -- func
-----------------------------------------------------------------------------------------
-- Mission objective "Sweep" spawn enemies
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveSweepSpawnEnemy()
	-- Is is usefull to have spawning functions separated from others to reduce code duplication and increase codea readability
	-- ALWAYS check for MOIDCount before spawning!!!
	if MovableMan:GetMOIDCount() <= self.MOIDLimit then
		local actor = nil;

		actor = CreateAHuman("Skeleton");
		actor.Pos = Vector(self.SweepZone:GetRandomPoint().X , 550);
		actor.Team = self.CPUTeam;
		actor.AIMode = Actor.AIMODE_SENTRY;
		MovableMan:AddActor(actor);
	end
end
-----------------------------------------------------------------------------------------
-- Mission objective "Assassinate" initialization
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveAssassinateInit()
	print ("Assasinate Init");
	self.AssassinateComplete = false;
	
	-- Remember fresh ronin in viriable so we can check if he is dead or not later
	self.Target = CreateAHuman("Dafred");
	self.Target.Pos = Vector(2064 , 559);
	self.Target.Team = self.CPUTeam;
	self.Target.AIMode = Actor.AIMODE_SENTRY;
	MovableMan:AddActor(self.Target);
end
-----------------------------------------------------------------------------------------
-- Mission objective "Assassinate" update state
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveAssassinateUpdate()
	if self.AssassinateComplete == false then
		-- Always check if actor stored in variable is still present before using it. You'll get lots of instant crashes 
		-- if you won't do this.
		if MovableMan:IsActor(self.Target) then
			self:AddObjectivePoint("Assassinate", self.Target.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
		else
			print ("Assasinate complete");
			self.AssassinateComplete = true;
			-- Nullify unused variable to avoid future glitches
			self.Target = nil;
		end
	end --if 
end -- func
-----------------------------------------------------------------------------------------
-- Mission objective "GetChip" initialization
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveGetChipInit()
	print ("Chip init");
	self.GetChipComplete = false;

	self:ObjectiveGetChipSpawnNewChip();
end
-----------------------------------------------------------------------------------------
-- Mission objective "GetChip" update state
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveGetChipUpdate()
	local currentstate = self.GetChipComplete;
	local chiplaying = false;
	local chipheld = false;
	self.GetChipComplete = true;
	
	-- We have the same situation with double looping here as with actors.
	for item in MovableMan.AddedItems do
		if item.PresetName == "Dummy Chip" then
			self:AddObjectivePoint("Pick chip!", item.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
			self.GetChipComplete = false;
			chiplaying = true;
		end -- if
	end -- for

	for item in MovableMan.Items do
		if item.PresetName == "Dummy Chip" then
			self:AddObjectivePoint("Pick chip!", item.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
			self.GetChipComplete = false;
			chiplaying = true;
		end -- if
	end -- for
	
	-- We need to check if we have a chip laying around or held by somebody. If we loose chip (thats more than possible)
	-- then our mission cannot be completed.
	for actor in MovableMan.Actors do
		if actor:HasObject("Dummy Chip") then
			chipheld = true;
		end
	end -- for
	
	-- Spawn new chip if we lost one.
	if chiplaying == false and chipheld == false and self.EvacuateComplete == false then
		print ("Chip Destroyed!");
		self:ObjectiveGetChipSpawnNewChip();
	end
	
	-- Don't forget about debug output. Here we can monitor chip objective state changes.
	if currentstate ~= self.GetChipComplete then
		if self.GetChipComplete == true then
			print ("GetChip complete");
		else	
			print ("Chip lost!");
		end -- if
	end -- if
end -- func
-----------------------------------------------------------------------------------------
-- Mission objective "GetChip" spawn chip
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveGetChipSpawnNewChip()
	print ("New Chip");

	local chip = CreateHeldDevice("Dummy Chip","MissionTut_04.rte");
	chip.Pos = Vector(2630 , 607)
	MovableMan:AddItem(chip);
end
-----------------------------------------------------------------------------------------
-- Mission objective "Evacuate" initialization
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveEvacuateInit()
	self.EvacuateComplete = false;
end
-----------------------------------------------------------------------------------------
-- Mission objective "Evacuate" update state
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveEvacuateUpdate()
	local wehaverocket = false;
	
	-- Check if we have a rocket to evacuate on. Give player a new one if not. You'll need to check for other
	-- ship's preset names in more complex missions with dropships and other ships.
	if self.GetChipComplete and self.AssassinateComplete and self.SweepComplete and not self.EvacuateComplete then
		for actor in MovableMan.Actors do
			if actor.PresetName == "Rocket MK2" and actor.Team == self.PlayerTeam then
				wehaverocket = true;
				self:AddObjectivePoint("Evacuate", actor.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWUP);
			end --if
		end -- for	
	
		if not wehaverocket then
			self:ObjectiveEvacuateNewRocket();
		end --if
	end --if
end -- func
-----------------------------------------------------------------------------------------
-- Mission objective "Evacuate" spawn new rocket
-----------------------------------------------------------------------------------------
function CampaignMission02:ObjectiveEvacuateNewRocket()
	print ("New Rocket");

	local rocket = CreateACRocket("Rocket MK2" , "Base.rte");
	rocket.Pos = Vector(3005 , -10);
	rocket.Team = self.PlayerTeam;
	rocket:SetControllerMode(Controller.CIM_AI, -1);

	MovableMan:AddActor(rocket);
end
-----------------------------------------------------------------------------------------
-- Enter orbit, check winning conditions
-----------------------------------------------------------------------------------------
function CampaignMission02:CraftEnteredOrbit()
	--If a craft leaves with the brain robot...
	if self.OrbitedCraft:HasObject("Brain Robot") then
		-- Team 1 wins!
		self.WinnerTeam = self.PlayerTeam;
		CF_MakeMissionAvailable("CampaignFramework.rte" , 3)
		ActivityMan:EndActivity();
		self.EvacuateComplete = true;
		-- Tell the script to stop running for brains as the game has ended.
		for player = 0, self.PlayerCount - 1 do
			self.braindead[player] = true;
		end
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------



































